module Calendar where

type Date    = (Day, Month, Year)
type Day     = Int -- day of month
type Month   = Int -- 1 January, 2 February, ...
type Year    = Int
type Dayname = Int -- 0 Sunday, 1 Monday, 2 Tuesday, ...

day :: Date -> Dayname
day (d,m,y) = (firstDayOfMonth m y + d - 1) `mod` 7

monthInfo :: Month -> Year -> (Dayname, Int)
monthInfo m y = zip (firstDaysOfYear y) (monthLengths y) !! (m-1)

firstDayOfMonth :: Month -> Year -> Dayname
firstDayOfMonth m y = firstDaysOfYear y !! (m-1)

firstDaysOfYear :: Year -> [Dayname]
firstDaysOfYear = take 12 . map (`mod` 7) . monthTotals

monthTotals :: Year -> [Int]
monthTotals y = scanl (+) (january1 y) (monthLengths y)

monthLengths :: Year -> [Int]
monthLengths y = [31,feb,31,30,31,30,31,31,30,31,30,31]
  where feb = if leap y then 29 else 28

leap :: Year -> Bool
leap y = if y `mod` 100 == 0 then (y `mod` 400 == 0)
                             else (y `mod` 4   == 0)

january1 :: Year -> Dayname
january1 y = (365*x + x `div` 4 - x `div` 100 + x `div` 400 + 1) `mod` 7
  where x = y - 1
